@@ -14,15 +14,19 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone import config
-from keystone.common import logging
+import os.path
+
 from keystone.catalog.backends import kvs
+from keystone.common import logging
+from keystone import config
 
 
 LOG = logging.getLogger(__name__)
 
 CONF = config.CONF
-config.register_str('template_file', group='catalog')
+config.register_str('template_file',
+                    default='default_catalog.templates',
+                    group='catalog')
 
 
 def parse_templates(template_lines):
@@ -91,7 +95,10 @@ class TemplatedCatalog(kvs.Catalog):
         if templates:
             self.templates = templates
         else:
-            self._load_templates(CONF.catalog.template_file)
+            template_file = CONF.catalog.template_file
+            if not os.path.exists(template_file):
+                template_file = CONF.find_file(template_file)
+            self._load_templates(template_file)
         super(TemplatedCatalog, self).__init__()
 
     def _load_templates(self, template_file):
