@@ -19,13 +19,14 @@ import uuid
 import ldap
 from ldap import filter as ldap_filter
 
+from keystone import clean
+from keystone.common import ldap as common_ldap
+from keystone.common.ldap import fakeldap
+from keystone.common import utils
 from keystone import config
 from keystone import exception
 from keystone import identity
-from keystone.common import ldap as common_ldap
-from keystone.common import utils
-from keystone.common.ldap import fakeldap
-from keystone.identity import models
+from keystone.common import models
 
 
 CONF = config.CONF
@@ -76,8 +77,12 @@ class Identity(identity.Driver):
         Expects the user object to have a password field and the tenant to be
         in the list of tenants on the user.
         """
-        user_ref = self._get_user(user_id)
-        if user_ref is None:
+        tenant_ref = None
+        metadata_ref = {}
+
+        try:
+            user_ref = self._get_user(user_id)
+        except exception.UserNotFound:
             raise AssertionError('Invalid user / password')
 
         try:
@@ -88,64 +93,99 @@ class Identity(identity.Driver):
         except Exception:
             raise AssertionError('Invalid user / password')
 
-        tenants = self.get_tenants_for_user(user_id)
-        if tenant_id and tenant_id not in tenants:
-            raise AssertionError('Invalid tenant')
+        if tenant_id is not None:
+            if tenant_id not in self.get_tenants_for_user(user_id):
+                raise AssertionError('Invalid tenant')
 
-        tenant_ref = self.get_tenant(tenant_id)
-        metadata_ref = {}
-        # TODO(termie): this should probably be made into a get roles call
-        #if tenant_ref:
-        #    metadata_ref =  self.get_metadata(user_id, tenant_id)
-        #else:
-        #    metadata_ref = {}
-        return  (_filter_user(user_ref), tenant_ref, metadata_ref)
+            try:
+                tenant_ref = self.get_tenant(tenant_id)
+                # TODO(termie): this should probably be made into a
+                #               get roles call
+                metadata_ref = self.get_metadata(user_id, tenant_id)
+            except exception.TenantNotFound:
+                tenant_ref = None
+                metadata_ref = {}
+            except exception.MetadataNotFound:
+                metadata_ref = {}
+
+        return (_filter_user(user_ref), tenant_ref, metadata_ref)
 
     def get_tenant(self, tenant_id):
-        return self.tenant.get(tenant_id)
+        try:
+            return self.tenant.get(tenant_id)
+        except exception.NotFound:
+            raise exception.TenantNotFound(tenant_id=tenant_id)
+
+    def get_tenants(self):
+        return self.tenant.get_all()
 
     def get_tenant_by_name(self, tenant_name):
-        return self.tenant.get_by_name(tenant_name)
+        try:
+            return self.tenant.get_by_name(tenant_name)
+        except exception.NotFound:
+            raise exception.TenantNotFound(tenant_id=tenant_name)
 
     def _get_user(self, user_id):
-        user_ref = self.user.get(user_id)
-        if not user_ref:
-            return None
-        return user_ref
+        try:
+            return self.user.get(user_id)
+        except exception.NotFound:
+            raise exception.UserNotFound(user_id=user_id)
 
     def get_user(self, user_id):
-        user_ref = self._get_user(user_id)
-        if (not user_ref):
-                return None
-        return _filter_user(user_ref)
+        return _filter_user(self._get_user(user_id))
+
+    def list_users(self):
+        return self.user.get_all()
 
     def get_user_by_name(self, user_name):
-        user_ref = self.user.get_by_name(user_name)
-        if not user_ref:
-            return None
-        return _filter_user(user_ref)
+        try:
+            return _filter_user(self.user.get_by_name(user_name))
+        except exception.NotFound:
+            raise exception.UserNotFound(user_id=user_name)
 
     def get_metadata(self, user_id, tenant_id):
         if not self.get_tenant(tenant_id) or not self.get_user(user_id):
             return {}
 
         metadata_ref = self.get_roles_for_user_and_tenant(user_id, tenant_id)
-        return metadata_ref or {}
+        if not metadata_ref:
+            return {}
+        return {'roles': metadata_ref}
 
     def get_role(self, role_id):
-        return self.role.get(role_id)
+        try:
+            return self.role.get(role_id)
+        except exception.NotFound:
+            raise exception.RoleNotFound(role_id=role_id)
+
+    def list_roles(self):
+        return self.role.get_all()
 
     # These should probably be part of the high-level API
+    # When this happens, then change TenantAPI.add_user to not ignore
+    # ldap.TYPE_OR_VALUE_EXISTS
     def add_user_to_tenant(self, tenant_id, user_id):
+        self.get_tenant(tenant_id)
+        self.get_user(user_id)
         return self.tenant.add_user(tenant_id, user_id)
 
     def get_tenants_for_user(self, user_id):
+        self.get_user(user_id)
         tenant_list = []
         for tenant in self.tenant.get_user_tenants(user_id):
             tenant_list.append(tenant['id'])
         return tenant_list
 
+    def get_tenant_users(self, tenant_id):
+        self.get_tenant(tenant_id)
+        user_list = []
+        for user in self.tenant.get_users(tenant_id):
+            user_list.append(user)
+        return user_list
+
     def get_roles_for_user_and_tenant(self, user_id, tenant_id):
+        self.get_user(user_id)
+        self.get_tenant(tenant_id)
         assignments = self.role.get_role_assignments(tenant_id)
         roles = []
         for assignment in assignments:
@@ -154,38 +194,84 @@ class Identity(identity.Driver):
         return roles
 
     def add_role_to_user_and_tenant(self, user_id, tenant_id, role_id):
+        self.get_user(user_id)
+        self.get_tenant(tenant_id)
+        self.get_role(role_id)
         self.role.add_user(role_id, user_id, tenant_id)
 
     # CRUD
     def create_user(self, user_id, user):
+        user['name'] = clean.user_name(user['name'])
         return self.user.create(user)
 
     def update_user(self, user_id, user):
+        if 'name' in user:
+            user['name'] = clean.user_name(user['name'])
         return self.user.update(user_id, user)
 
     def create_tenant(self, tenant_id, tenant):
+        tenant['name'] = clean.tenant_name(tenant['name'])
         data = tenant.copy()
         if 'id' not in data or data['id'] is None:
             data['id'] = str(uuid.uuid4().hex)
         return self.tenant.create(tenant)
 
     def update_tenant(self, tenant_id, tenant):
+        if 'name' in tenant:
+            tenant['name'] = clean.tenant_name(tenant['name'])
         return self.tenant.update(tenant_id, tenant)
 
     def create_metadata(self, user_id, tenant_id, metadata):
         return {}
 
     def create_role(self, role_id, role):
-        if self.get_role(role_id):
+        try:
+            self.get_role(role_id)
+        except exception.NotFound:
+            pass
+        else:
             msg = 'Duplicate ID, %s.' % role_id
             raise exception.Conflict(type='role', details=msg)
-        if self.role.get_by_name(role['name']):
+
+        try:
+            self.role.get_by_name(role['name'])
+        except exception.NotFound:
+            pass
+        else:
             msg = 'Duplicate name, %s.' % role['name']
             raise exception.Conflict(type='role', details=msg)
+
         return self.role.create(role)
 
     def delete_role(self, role_id):
-        return self.role.delete(role_id)
+        try:
+            return self.role.delete(role_id)
+        except ldap.NO_SUCH_OBJECT:
+            raise exception.RoleNotFound(role_id=role_id)
+
+    def delete_tenant(self, tenant_id):
+        try:
+            return self.tenant.delete(tenant_id)
+        except ldap.NO_SUCH_OBJECT:
+            raise exception.TenantNotFound(tenant_id=tenant_id)
+
+    def delete_user(self, user_id):
+        try:
+            return self.user.delete(user_id)
+        except ldap.NO_SUCH_OBJECT:
+            raise exception.UserNotFound(user_id=user_id)
+
+    def remove_role_from_user_and_tenant(self, user_id, tenant_id, role_id):
+        return self.role.delete_user(role_id, user_id, tenant_id)
+
+    def remove_user_from_tenant(self, tenant_id, user_id):
+        self.get_user(user_id)
+        self.get_tenant(tenant_id)
+        return self.tenant.remove_user(tenant_id, user_id)
+
+    def update_role(self, role_id, role):
+        self.get_role(role_id)
+        self.role.update(role_id, role)
 
 
 # TODO(termie): remove this and move cross-api calls into driver
@@ -247,7 +333,7 @@ class UserApi(common_ldap.BaseLdap, ApiShimMixin):
     DEFAULT_OBJECTCLASS = 'inetOrgPerson'
     options_name = 'user'
     attribute_mapping = {'password': 'userPassword',
-                         #'email': 'mail',
+                         'email': 'mail',
                          'name': 'sn'}
 
     # NOTE(ayoung): The RFC based schemas don't have a way to indicate
@@ -255,13 +341,21 @@ class UserApi(common_ldap.BaseLdap, ApiShimMixin):
     # be part of any objectclass.
     # in the future, we need to provide a way for the end user to
     # indicate the field to use and what it indicates
-    attribute_ignore = ['tenant_id', 'enabled', 'tenants']
+    attribute_ignore = ['tenantId', 'enabled', 'tenants']
     model = models.User
 
     def __init__(self, conf):
         super(UserApi, self).__init__(conf)
+        self.attribute_mapping['name'] = conf.ldap.user_name_attribute
         self.api = ApiShim(conf)
 
+    def get(self, id, filter=None):
+        """Replaces exception.NotFound with exception.UserNotFound."""
+        try:
+            return super(UserApi, self).get(id, filter)
+        except exception.NotFound:
+            raise exception.UserNotFound(user_id=id)
+
     def get_by_name(self, name, filter=None):
         users = self.get_all('(%s=%s)' %
                              (self.attribute_mapping['name'],
@@ -269,7 +363,7 @@ class UserApi(common_ldap.BaseLdap, ApiShimMixin):
         try:
             return users[0]
         except IndexError:
-            return None
+            raise exception.UserNotFound(user_id=name)
 
     def create(self, values):
         self.affirm_unique(values)
@@ -282,11 +376,13 @@ class UserApi(common_ldap.BaseLdap, ApiShimMixin):
 
     def update(self, id, values):
         if values['id'] != id:
-            return None
-        old_obj = self.get(id)
+            raise exception.ValidationError('Cannot change user ID')
+        try:
+            old_obj = self.get(id)
+        except exception.NotFound:
+            raise exception.UserNotFound(user_id=id)
         if old_obj.get('name') != values['name']:
-            raise exception.Error('Changing Name not permitted')
-
+            raise exception.Conflict('Cannot change user name')
         try:
             new_tenant = values['tenant_id']
         except KeyError:
@@ -303,7 +399,7 @@ class UserApi(common_ldap.BaseLdap, ApiShimMixin):
 
     def delete(self, id):
         user = self.get(id)
-        if user.tenant_id:
+        if hasattr(user, 'tenant_id'):
             self.tenant_api.remove_user(user.tenant_id, id)
 
         super(UserApi, self).delete(id)
@@ -315,8 +411,8 @@ class UserApi(common_ldap.BaseLdap, ApiShimMixin):
             self.role_api.rolegrant_delete(ref.id)
 
     def get_by_email(self, email):
-        users = self.get_all('(mail=%s)' % \
-                            (ldap_filter.escape_filter_chars(email),))
+        users = self.get_all('(mail=%s)' %
+                             (ldap_filter.escape_filter_chars(email),))
         try:
             return users[0]
         except IndexError:
@@ -352,10 +448,10 @@ class UserApi(common_ldap.BaseLdap, ApiShimMixin):
                               limit,
                               self.tenant_api.get_users(tenant_id, role_id))
 
-    def users_get_by_tenant_get_page_markers(self, tenant_id,
-        role_id, marker, limit):
+    def users_get_by_tenant_get_page_markers(self, tenant_id, role_id, marker,
+                                             limit):
         return self._get_page_markers(
-                marker, limit, self.tenant_api.get_users(tenant_id, role_id))
+            marker, limit, self.tenant_api.get_users(tenant_id, role_id))
 
     def check_password(self, user_id, password):
         user = self.get(user_id)
@@ -370,15 +466,24 @@ class TenantApi(common_ldap.BaseLdap, ApiShimMixin):
     DEFAULT_ID_ATTR = 'cn'
     DEFAULT_MEMBER_ATTRIBUTE = 'member'
     options_name = 'tenant'
-    attribute_mapping = {'description': 'desc', 'name': 'ou'}
+    attribute_mapping = {'name': 'ou', 'tenantId': 'cn'}
+    attribute_ignore = ['enabled']
     model = models.Tenant
 
     def __init__(self, conf):
         super(TenantApi, self).__init__(conf)
         self.api = ApiShim(conf)
+        self.attribute_mapping['name'] = conf.ldap.tenant_name_attribute
         self.member_attribute = (getattr(conf.ldap, 'tenant_member_attribute')
                                  or self.DEFAULT_MEMBER_ATTRIBUTE)
 
+    def get(self, id, filter=None):
+        """Replaces exception.NotFound with exception.TenantNotFound."""
+        try:
+            return super(TenantApi, self).get(id, filter)
+        except exception.NotFound:
+            raise exception.TenantNotFound(tenant_id=id)
+
     def get_by_name(self, name, filter=None):  # pylint: disable=W0221,W0613
         search_filter = ('(%s=%s)'
                          % (self.attribute_mapping['name'],
@@ -387,13 +492,12 @@ class TenantApi(common_ldap.BaseLdap, ApiShimMixin):
         try:
             return tenants[0]
         except IndexError:
-            return None
+            raise exception.TenantNotFound(tenant_id=name)
 
     def create(self, values):
         self.affirm_unique(values)
-
         data = values.copy()
-        if 'id' not in data or data['id'] is None:
+        if data.get('id') is None:
             data['id'] = uuid.uuid4().hex
         return super(TenantApi, self).create(data)
 
@@ -412,7 +516,7 @@ class TenantApi(common_ldap.BaseLdap, ApiShimMixin):
 
     def list_for_user_get_page_markers(self, user, marker, limit):
         return self._get_page_markers(
-                marker, limit, self.get_user_tenants(user['id']))
+            marker, limit, self.get_user_tenants(user['id']))
 
     def is_empty(self, id):
         tenant = self._ldap_get(id)
@@ -428,17 +532,27 @@ class TenantApi(common_ldap.BaseLdap, ApiShimMixin):
 
     def add_user(self, tenant_id, user_id):
         conn = self.get_connection()
-        conn.modify_s(self._id_to_dn(tenant_id),
-                      [(ldap.MOD_ADD,
-                        self.member_attribute,
-                        self.user_api._id_to_dn(user_id))])
+        try:
+            conn.modify_s(
+                self._id_to_dn(tenant_id),
+                [(ldap.MOD_ADD,
+                  self.member_attribute,
+                  self.user_api._id_to_dn(user_id))])
+        except ldap.TYPE_OR_VALUE_EXISTS:
+            # As adding a user to a tenant is done implicitly in several
+            # places, and is not part of the exposed API, it's easier for us to
+            # just ignore this instead of raising exception.Conflict.
+            pass
 
     def remove_user(self, tenant_id, user_id):
         conn = self.get_connection()
-        conn.modify_s(self._id_to_dn(tenant_id),
-                      [(ldap.MOD_DELETE,
-                        self.member_attribute,
-                        self.user_api._id_to_dn(user_id))])
+        try:
+            conn.modify_s(self._id_to_dn(tenant_id),
+                          [(ldap.MOD_DELETE,
+                            self.member_attribute,
+                            self.user_api._id_to_dn(user_id))])
+        except ldap.NO_SUCH_ATTRIBUTE:
+            raise exception.NotFound(user_id)
 
     def get_users(self, tenant_id, role_id=None):
         tenant = self._ldap_get(tenant_id)
@@ -461,23 +575,19 @@ class TenantApi(common_ldap.BaseLdap, ApiShimMixin):
         super(TenantApi, self).delete(id)
 
     def update(self, id, values):
-        old_obj = self.get(id)
+        try:
+            old_obj = self.get(id)
+        except exception.NotFound:
+            raise exception.TenantNotFound(tenant_id=id)
         if old_obj['name'] != values['name']:
-            raise exception.Error('Changing Name not permitted')
+            msg = 'Changing Name not supported by LDAP'
+            raise exception.NotImplemented(message=msg)
         super(TenantApi, self).update(id, values, old_obj)
 
 
 class UserRoleAssociation(object):
     """Role Grant model."""
 
-    hints = {
-        'contract_attributes': ['id', 'role_id', 'user_id', 'tenant_id'],
-        'types': [('user_id', basestring), ('tenant_id', basestring)],
-        'maps': {'userId': 'user_id',
-                 'roleId': 'role_id',
-                 'tenantId': 'tenant_id'}
-    }
-
     def __init__(self, user_id=None, role_id=None, tenant_id=None,
                  *args, **kw):
         self.user_id = str(user_id)
@@ -552,12 +662,9 @@ class RoleApi(common_ldap.BaseLdap, ApiShimMixin):
         try:
             return roles[0]
         except IndexError:
-            return None
+            raise exception.RoleNotFound(role_id=name)
 
     def add_user(self, role_id, user_id, tenant_id=None):
-        user = self.user_api.get(user_id)
-        if user is None:
-            raise exception.UserNotFound(user_id=user_id)
         role_dn = self._subrole_id_to_dn(role_id, tenant_id)
         conn = self.get_connection()
         user_dn = self.user_api._id_to_dn(user_id)
@@ -565,8 +672,9 @@ class RoleApi(common_ldap.BaseLdap, ApiShimMixin):
             conn.modify_s(role_dn, [(ldap.MOD_ADD,
                                      self.member_attribute, user_dn)])
         except ldap.TYPE_OR_VALUE_EXISTS:
-            raise exception.Error('User %s already has role %s in tenant %s'
-                                  % (user_id, role_id, tenant_id))
+            msg = ('User %s already has role %s in tenant %s'
+                   % (user_id, role_id, tenant_id))
+            raise exception.Conflict(type='role grant', details=msg)
         except ldap.NO_SUCH_OBJECT:
             if tenant_id is None or self.get(role_id) is None:
                 raise Exception("Role %s not found" % (role_id,))
@@ -582,10 +690,33 @@ class RoleApi(common_ldap.BaseLdap, ApiShimMixin):
                     raise inst
 
         return UserRoleAssociation(
-                id=self._create_ref(role_id, tenant_id, user_id),
-                role_id=role_id,
-                user_id=user_id,
-                tenant_id=tenant_id)
+            id=self._create_ref(role_id, tenant_id, user_id),
+            role_id=role_id,
+            user_id=user_id,
+            tenant_id=tenant_id)
+
+    def delete_user(self, role_id, user_id, tenant_id):
+        role_dn = self._subrole_id_to_dn(role_id, tenant_id)
+        conn = self.get_connection()
+        user_dn = self.user_api._id_to_dn(user_id)
+        try:
+            conn.modify_s(role_dn, [(ldap.MOD_DELETE,
+                                     self.member_attribute, user_dn)])
+        except ldap.NO_SUCH_OBJECT:
+            if tenant_id is None or self.get(role_id) is None:
+                raise exception.RoleNotFound(role_id=roll_id)
+            attrs = [('objectClass', [self.object_class]),
+                     (self.member_attribute, [user_dn])]
+
+            if self.use_dumb_member:
+                attrs[1][1].append(self.DUMB_MEMBER_DN)
+            try:
+                conn.add_s(role_dn, attrs)
+            except Exception as inst:
+                raise inst
+
+        except ldap.NO_SUCH_ATTRIBUTE:
+            raise exception.UserNotFound(user_id=user_id)
 
     def get_by_service(self, service_id):
         roles = self.get_all('(service_id=%s)' %
@@ -620,10 +751,10 @@ class RoleApi(common_ldap.BaseLdap, ApiShimMixin):
                 user_id = self.user_api._dn_to_id(user_dn)
                 role_id = self._dn_to_id(role_dn)
                 res.append(UserRoleAssociation(
-                        id=self._create_ref(role_id, tenant_id, user_id),
-                        user_id=user_id,
-                        role_id=role_id,
-                        tenant_id=tenant_id))
+                    id=self._create_ref(role_id, tenant_id, user_id),
+                    user_id=user_id,
+                    role_id=role_id,
+                    tenant_id=tenant_id))
 
         return res
 
@@ -631,10 +762,9 @@ class RoleApi(common_ldap.BaseLdap, ApiShimMixin):
         user_dn = self.user_api._id_to_dn(user_id)
         roles = self.get_all('(%s=%s)' % (self.member_attribute, user_dn))
         return [UserRoleAssociation(
-                    id=self._create_ref(role.id, None, user_id),
-                    role_id=role.id,
-                    user_id=user_id)
-                for role in roles]
+                id=self._create_ref(role.id, None, user_id),
+                role_id=role.id,
+                user_id=user_id) for role in roles]
 
     def list_tenant_roles_for_user(self, user_id, tenant_id=None):
         conn = self.get_connection()
@@ -653,10 +783,10 @@ class RoleApi(common_ldap.BaseLdap, ApiShimMixin):
             for role_dn, _ in roles:
                 role_id = self._dn_to_id(role_dn)
                 res.append(UserRoleAssociation(
-                        id=self._create_ref(role_id, tenant_id, user_id),
-                        user_id=user_id,
-                        role_id=role_id,
-                        tenant_id=tenant_id))
+                    id=self._create_ref(role_id, tenant_id, user_id),
+                    user_id=user_id,
+                    role_id=role_id,
+                    tenant_id=tenant_id))
         else:
             try:
                 roles = conn.search_s(self.tenant_api.tree_dn,
@@ -670,10 +800,10 @@ class RoleApi(common_ldap.BaseLdap, ApiShimMixin):
                 role_id = self._dn_to_id(role_dn)
                 tenant_id = ldap.dn.str2dn(role_dn)[1][0][1]
                 res.append(UserRoleAssociation(
-                        id=self._create_ref(role_id, tenant_id, user_id),
-                        user_id=user_id,
-                        role_id=role_id,
-                        tenant_id=tenant_id))
+                    id=self._create_ref(role_id, tenant_id, user_id),
+                    user_id=user_id,
+                    role_id=role_id,
+                    tenant_id=tenant_id))
         return res
 
     def rolegrant_get(self, id):
@@ -785,10 +915,10 @@ class RoleApi(common_ldap.BaseLdap, ApiShimMixin):
                 ldap_role_id = self._dn_to_id(role_dn)
                 if role_id == ldap_role_id:
                     res = UserRoleAssociation(
-                            id=self._create_ref(role_id, tenant_id, user_id),
-                            user_id=user_id,
-                            role_id=role_id,
-                            tenant_id=tenant_id)
+                        id=self._create_ref(role_id, tenant_id, user_id),
+                        user_id=user_id,
+                        role_id=role_id,
+                        tenant_id=tenant_id)
                     return res
         else:
             try:
@@ -803,7 +933,20 @@ class RoleApi(common_ldap.BaseLdap, ApiShimMixin):
             for role in roles:
                 if role.id == role_id:
                     return UserRoleAssociation(
-                            id=self._create_ref(role.id, None, user_id),
-                            role_id=role.id,
-                            user_id=user_id)
+                        id=self._create_ref(role.id, None, user_id),
+                        role_id=role.id,
+                        user_id=user_id)
         return None
+
+    def update(self, role_id, role):
+        if role['id'] != role_id:
+            raise exception.ValidationError('Cannot change role ID')
+        try:
+            old_name = self.get_by_name(role['name'])
+            raise exception.Conflict('Cannot duplicate name %s' % role['name'])
+        except exception.NotFound:
+            pass
+        try:
+            super(RoleApi, self).update(id, role)
+        except exception.NotFound:
+            raise exception.UserNotFound(user_id=id)
