@@ -14,14 +14,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import json
-
-import webob.exc
-
-from keystone import config
-from keystone import exception
 from keystone.common import serializer
 from keystone.common import wsgi
+from keystone import config
+from keystone import exception
+from keystone.openstack.common import jsonutils
 
 
 CONF = config.CONF
@@ -98,21 +95,25 @@ class JsonBodyMiddleware(wsgi.Middleware):
 
     """
     def process_request(self, request):
-        # Ignore unrecognized content types. Empty string indicates
-        # the client did not explicitly set the header
-        if not request.content_type in ('application/json', ''):
-            return
-
+        # Abort early if we don't have any work to do
         params_json = request.body
         if not params_json:
             return
 
+        # Reject unrecognized content types. Empty string indicates
+        # the client did not explicitly set the header
+        if not request.content_type in ('application/json', ''):
+            e = exception.ValidationError(attribute='application/json',
+                                          target='Content-Type header')
+            return wsgi.render_exception(e)
+
         params_parsed = {}
         try:
-            params_parsed = json.loads(params_json)
+            params_parsed = jsonutils.loads(params_json)
         except ValueError:
-            msg = 'Malformed json in request body'
-            raise webob.exc.HTTPBadRequest(explanation=msg)
+            e = exception.ValidationError(attribute='valid JSON',
+                                          target='request body')
+            return wsgi.render_exception(e)
         finally:
             if not params_parsed:
                 params_parsed = {}
@@ -136,7 +137,7 @@ class XmlBodyMiddleware(wsgi.Middleware):
         incoming_xml = 'application/xml' in str(request.content_type)
         if incoming_xml and request.body:
             request.content_type = 'application/json'
-            request.body = json.dumps(serializer.from_xml(request.body))
+            request.body = jsonutils.dumps(serializer.from_xml(request.body))
 
     def process_response(self, request, response):
         """Transform the response from JSON to XML."""
@@ -144,7 +145,22 @@ class XmlBodyMiddleware(wsgi.Middleware):
         if outgoing_xml and response.body:
             response.content_type = 'application/xml'
             try:
-                response.body = serializer.to_xml(json.loads(response.body))
+                body_obj = jsonutils.loads(response.body)
+                response.body = serializer.to_xml(body_obj)
             except:
                 raise exception.Error(message=response.body)
         return response
+
+
+class NormalizingFilter(wsgi.Middleware):
+    """Middleware filter to handle URL normalization."""
+
+    def process_request(self, request):
+        """Normalizes URLs."""
+        # Removes a trailing slash from the given path, if any.
+        if (len(request.environ['PATH_INFO']) > 1 and
+                request.environ['PATH_INFO'][-1] == '/'):
+            request.environ['PATH_INFO'] = request.environ['PATH_INFO'][:-1]
+        # Rewrites path to root if no path is given.
+        elif not request.environ['PATH_INFO']:
+            request.environ['PATH_INFO'] = '/'
