@@ -16,9 +16,12 @@
 
 import copy
 import datetime
+import hashlib
 
+from keystone.common import cms
 from keystone.common import sql
 from keystone import exception
+from keystone.openstack.common import timeutils
 from keystone import token
 
 
@@ -27,6 +30,7 @@ class TokenModel(sql.ModelBase, sql.DictBase):
     id = sql.Column(sql.String(64), primary_key=True)
     expires = sql.Column(sql.DateTime(), default=None)
     extra = sql.Column(sql.JsonBlob())
+    valid = sql.Column(sql.Boolean(), default=True)
 
     @classmethod
     def from_dict(cls, token_dict):
@@ -49,21 +53,31 @@ class Token(sql.Base, token.Driver):
     # Public interface
     def get_token(self, token_id):
         session = self.get_session()
-        token_ref = session.query(TokenModel).filter_by(id=token_id).first()
+        token_ref = session.query(TokenModel)\
+            .filter_by(id=self.token_to_key(token_id),
+                       valid=True).first()
         now = datetime.datetime.utcnow()
         if token_ref and (not token_ref.expires or now < token_ref.expires):
             return token_ref.to_dict()
         else:
             raise exception.TokenNotFound(token_id=token_id)
 
+    def token_to_key(self, token_id):
+        if len(token_id) > cms.UUID_TOKEN_LENGTH:
+            hash = hashlib.md5()
+            hash.update(token_id)
+            return hash.hexdigest()
+        else:
+            return token_id
+
     def create_token(self, token_id, data):
         data_copy = copy.deepcopy(data)
         if 'expires' not in data_copy:
             data_copy['expires'] = self._get_default_expire_time()
 
         token_ref = TokenModel.from_dict(data_copy)
-        token_ref.id = token_id
-
+        token_ref.id = self.token_to_key(token_id)
+        token_ref.valid = True
         session = self.get_session()
         with session.begin():
             session.add(token_ref)
@@ -72,22 +86,22 @@ class Token(sql.Base, token.Driver):
 
     def delete_token(self, token_id):
         session = self.get_session()
-        token_ref = session.query(TokenModel)\
-                                .filter_by(id=token_id)\
-                                .first()
-        if not token_ref:
-            raise exception.TokenNotFound(token_id=token_id)
-
+        key = self.token_to_key(token_id)
         with session.begin():
-            session.delete(token_ref)
+            token_ref = session.query(TokenModel).filter_by(id=key,
+                                                            valid=True).first()
+            if not token_ref:
+                raise exception.TokenNotFound(token_id=token_id)
+            token_ref.valid = False
             session.flush()
 
     def list_tokens(self, user_id, tenant_id=None):
         session = self.get_session()
         tokens = []
-        now = datetime.datetime.utcnow()
+        now = timeutils.utcnow()
         for token_ref in session.query(TokenModel)\
-                                      .filter(TokenModel.expires > now):
+                                .filter(TokenModel.expires > now)\
+                                .filter_by(valid=True):
             token_ref_dict = token_ref.to_dict()
             if 'user' not in token_ref_dict:
                 continue
@@ -100,3 +114,17 @@ class Token(sql.Base, token.Driver):
                     continue
             tokens.append(token_ref['id'])
         return tokens
+
+    def list_revoked_tokens(self):
+        session = self.get_session()
+        tokens = []
+        now = timeutils.utcnow()
+        for token_ref in session.query(TokenModel)\
+                                .filter(TokenModel.expires > now)\
+                                .filter_by(valid=False):
+            record = {
+                'id': token_ref['id'],
+                'expires': token_ref['expires'],
+            }
+            tokens.append(record)
+        return tokens
